@@ -28,12 +28,17 @@ class Authenticator(dns_common.DNSAuthenticator):
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
         self.credentials = None
-        self.base_domain = None
 
     @classmethod
     def add_parser_arguments(cls, add):  # pylint: disable=arguments-differ
         super(Authenticator, cls).add_parser_arguments(add)
         add("credentials", help="TencentCloud credentials INI file.")
+        add(
+            "debug",
+            help="turn on debug mode (print some debug info)",
+            type=bool,
+            default=False,
+        )
 
     # pylint: disable=no-self-use
     def more_info(self):  # pylint: disable=missing-function-docstring
@@ -51,9 +56,20 @@ class Authenticator(dns_common.DNSAuthenticator):
         if not v:
             raise errors.PluginError("{} is required".format(arg))
 
+    def chk_base_domain(self, base_domain, validation_name):
+        if not validation_name.endswith("." + base_domain):
+            raise errors.PluginError(
+                "validation_name not ends with base domain name, please report to dev. "
+                f"real_domain: {base_domain}, validation_name: {validation_name}"
+            )
+
     def determine_base_domain(self, domain):
+        if self.conf("debug"):
+            print("finding base domain")
         client = TencentCloudClient(
-            self.credentials.conf("secret_id"), self.credentials.conf("secret_key")
+            self.credentials.conf("secret_id"),
+            self.credentials.conf("secret_key"),
+            self.conf("debug"),
         )
         segments = domain.split(".")
         tried = []
@@ -63,12 +79,14 @@ class Authenticator(dns_common.DNSAuthenticator):
             tried.append(dt)
             i -= 1
             try:
-                client.get_record_list(dt)
+                resp = client.get_record_list(dt)
             # if error, we don't seem to own this domain
             except APIException as _:
                 continue
-            return dt, tried
-        return None, tried
+            return dt, resp["records"]
+        raise errors.PluginError(
+            "failed to determine base domain, please report to dev. " f"Tried: {tried}"
+        )
 
     # pylint: enable=no-self-use
 
@@ -81,48 +99,32 @@ class Authenticator(dns_common.DNSAuthenticator):
         )
 
     def _perform(self, domain, validation_name, validation):
+        if self.conf("debug"):
+            print("perform", domain, validation_name, validation)
         client = TencentCloudClient(
-            self.credentials.conf("secret_id"), self.credentials.conf("secret_key")
+            self.credentials.conf("secret_id"),
+            self.credentials.conf("secret_key"),
+            self.conf("debug"),
         )
-        base_domain, tried = self.determine_base_domain(domain)
-        if base_domain is None:
-            raise ValueError(
-                "failed to determine base domain, please report to dev. "
-                f"Tried: {tried}"
-            )
-        self.base_domain = base_domain
-        if not validation_name.endswith("." + base_domain):
-            raise ValueError(
-                "validation_name not ends with base domain name, please report to dev. "
-                f"real_domain: {base_domain}, validation_name: {validation_name}"
-            )
+        base_domain, _ = self.determine_base_domain(domain)
+        self.chk_base_domain(base_domain, validation_name)
+
         sub_domain = validation_name[: -(len(base_domain) + 1)]
-        resp = client.get_record_list(base_domain, sub_domain)
-        rid = None
-        if int(resp["info"]["record_total"]) > 0:
-            rid = resp["records"][0]["id"]
-        if rid is None:
-            client.get_record_create(base_domain, sub_domain, "TXT", validation)
-        else:
-            client.get_record_modify(base_domain, rid, sub_domain, "TXT", validation)
+        resp = client.get_record_create(base_domain, sub_domain, "TXT", validation)
 
     def _cleanup(self, domain, validation_name, validation):
+        if self.conf("debug"):
+            print("cleanup", domain, validation_name, validation)
         client = TencentCloudClient(
-            self.credentials.conf("secret_id"), self.credentials.conf("secret_key")
+            self.credentials.conf("secret_id"),
+            self.credentials.conf("secret_key"),
+            self.conf("debug"),
         )
-        if self.base_domain is None:
-            return
-        base_domain = self.base_domain
-        sub_domain = validation_name[: -(len(base_domain) + 1)]
-        resp = client.get_record_list(base_domain, sub_domain)
-        rid = None
-        if int(resp["info"]["record_total"]) > 0:
-            rid = resp["records"][0]["id"]
-        if rid is None:
-            raise errors.PluginError(
-                "could not find record in cleanup: {}".format(validation_name)
-            )
-        client.get_record_delete(base_domain, rid)
+        base_domain, records = self.determine_base_domain(domain)
+        self.chk_base_domain(base_domain, validation_name)
+        for rec in records:
+            if rec["type"] == "TXT" and rec["value"] == validation:
+                client.get_record_delete(base_domain, rec["id"])
 
 
 class APIException(Exception):
@@ -140,8 +142,9 @@ class TencentCloudClient:
     endpoint = "cns.api.qcloud.com/v2/index.php"
     url = "https://" + endpoint
 
-    def __init__(self, secret_id, secret_key):
+    def __init__(self, secret_id, secret_key, debug=False):
         self.cred = self.Cred(secret_id, secret_key)
+        self.debug = debug
 
     def _expand_params(self, action, params):
         params.update(
@@ -166,6 +169,8 @@ class TencentCloudClient:
 
     def mk_get_req(self, action, params):
         self._expand_params(action, params)
+        if self.debug:
+            print(f"\x1b[32;1m{action}\x1b[0m params: {params}")
         signature = self._mk_sign("GET", params)
         params["Signature"] = signature
         full_url = (
@@ -174,6 +179,8 @@ class TencentCloudClient:
             + "&".join([quote(str(k)) + "=" + quote(str(v)) for k, v in params.items()])
         )
         rj = json.loads(urlopen(full_url).read().decode())
+        if self.debug:
+            print(f"\x1b[31;1m{action}\x1b[31;0m resp: {rj}")
         if rj["code"] == 0:
             if "data" in rj:  # only exception is delete
                 return rj["data"]
